package ch.zhaw.pm2.multichat.client;

import ch.zhaw.pm2.multichat.client.ClientConnectionHandler.State;
import ch.zhaw.pm2.multichat.protocol.ChatProtocolException;
import ch.zhaw.pm2.multichat.protocol.NetworkHandler;
import javafx.application.Platform;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javafx.scene.layout.Pane;
import javafx.stage.WindowEvent;

import java.io.IOException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import static ch.zhaw.pm2.multichat.client.ClientConnectionHandler.State.*;

public class ChatWindowController {
    private ClientConnectionHandler connectionHandler;
    private ClientMessageList messages;

    private final WindowCloseHandler windowCloseHandler = new WindowCloseHandler();

    @FXML private Pane rootPane;
    @FXML private TextField serverAddressField;
    @FXML private TextField serverPortField;
    @FXML private TextField userNameField;
    @FXML private TextField messageField;
    @FXML private TextArea messageArea;
    @FXML private Button connectButton;
    @FXML private Button sendButton;
    @FXML private TextField filterValue;


    @FXML
    public void initialize() {
        serverAddressField.setText(NetworkHandler.DEFAULT_ADDRESS.getCanonicalHostName());
        serverPortField.setText(String.valueOf(NetworkHandler.DEFAULT_PORT));
        stateChanged(NEW);
    }

    public void setMessages(ClientMessageList messages) {
        this.messages = messages;
    }

    private void applicationClose() {
        connectionHandler.setState(DISCONNECTED);
    }

    @FXML
    private void toggleConnection () {
        if (connectionHandler == null || connectionHandler.getState() != CONNECTED) {
            connect();
        } else {
            disconnect();
        }
    }

    private void connect() {
        try {
            messages.clear(); // clear message list
            redrawMessageList();
            startConnectionHandler();
            connectionHandler.connect();
        } catch(ChatProtocolException | IOException e) {
            addError(e.getMessage());
        }
    }

    private void disconnect() {
        if (connectionHandler == null) {
            addError("No connection handler");
            return;
        }
        try {
            connectionHandler.disconnect();
        } catch (ChatProtocolException e) {
            addError(e.getMessage());
        }
    }

    @FXML
    private void message() {
        String messageString = messageField.getText().strip();
        try {
            if (connectionHandler == null) {
                addError("No connection handler");
            } else if (!connectionHandler.message(messageString)) {
                addError("Not a valid message format.");
            } else {
                messageField.clear();
            }
        } catch (ChatProtocolException e) {
            addError(e.getMessage());
        }
    }

    @FXML
    private void applyFilter( ) {
    	this.redrawMessageList();
    }

    private void startConnectionHandler() throws IOException {
        String userName = userNameField.getText();
        String serverAddress = serverAddressField.getText();
        int serverPort = Integer.parseInt(serverPortField.getText());
        connectionHandler = new ClientConnectionHandler(
            NetworkHandler.openConnection(serverAddress, serverPort), userName,
            this);
        new Thread(connectionHandler).start();

        // register window close handler
        rootPane.getScene().getWindow().addEventHandler(WindowEvent.WINDOW_CLOSE_REQUEST, windowCloseHandler);
    }

    private void terminateConnectionHandler() {
        // unregister window close handler
        rootPane.getScene().getWindow().removeEventHandler(WindowEvent.WINDOW_CLOSE_REQUEST, windowCloseHandler);
        if (connectionHandler != null) {
            connectionHandler.stopReceiving();
            connectionHandler = null;
        }
    }

    public void stateChanged(State newState) {
        // update UI (need to be run in UI thread: see Platform.runLater())
        Platform.runLater(new Runnable() {
            @Override
            public void run() {
                connectButton.setText((newState == CONNECTED || newState == CONFIRM_DISCONNECT) ? "Disconnect" : "Connect");
            }
        });
        if (newState == DISCONNECTED) {
            terminateConnectionHandler();
        }
    }

    public void setUserName(String userName) {
        Platform.runLater(new Runnable() {
            @Override
            public void run() {
                userNameField.setText(userName);
            }
        });
    }

    public void setServerAddress(String serverAddress) {
        Platform.runLater(new Runnable() {
            @Override
            public void run() {
                serverAddressField.setText(serverAddress);
            }
        });
    }

    public void setServerPort(int serverPort) {
        Platform.runLater(new Runnable() {
            @Override
            public void run() {
                serverPortField.setText(Integer.toString(serverPort));
            }
        });
    }

    public void addMessage(String sender, String receiver, String message) {
        messages.addMessage(new Message(Message.MessageType.MESSAGE, sender, receiver, message));
        this.redrawMessageList();
    }

    public void addInfo(String message) {
        messages.addMessage(new Message(Message.MessageType.INFO, null, null, message));
        this.redrawMessageList();
    }

    public void addError(String message) {
        messages.addMessage(new Message(Message.MessageType.ERROR, null, null, message));
        this.redrawMessageList();
    }

    private void redrawMessageList() {
        this.messageArea.clear();
        Platform.runLater(() -> this.messageArea.setText(messages.getFilteredMessages(filterValue.getText().strip())));
    }

    class WindowCloseHandler implements EventHandler<WindowEvent> {
        public void handle(WindowEvent event) {
            applicationClose();
        }

    }

}
